package com.company.util;

import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;

import org.jbpm.process.audit.JPAAuditLogService;
import org.jbpm.runtime.manager.impl.RuntimeEnvironmentBuilder;
import org.kie.api.io.ResourceType;
import org.kie.api.runtime.KieSession;
import org.kie.api.runtime.manager.RuntimeEngine;
import org.kie.api.runtime.manager.RuntimeManager;
import org.kie.api.runtime.manager.RuntimeManagerFactory;
import org.kie.api.task.TaskService;
import org.kie.internal.io.ResourceFactory;
import org.kie.internal.runtime.manager.RuntimeEnvironment;
import org.kie.internal.runtime.manager.context.EmptyContext;
import org.kie.internal.task.api.InternalTaskService;
import org.kie.internal.task.api.UserGroupCallback;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class PerRequestRuntimeManager {

    private static RuntimeManager runtimeManager;

    private static volatile PerRequestRuntimeManager instance = null;
    private static final Logger logger = LoggerFactory.getLogger(PerRequestRuntimeManager.class);

    public void setRuntimeManager(RuntimeManager myRuntimeManager) {
        runtimeManager = myRuntimeManager;
    }

    private PerRequestRuntimeManager() {
    }

    public static PerRequestRuntimeManager getInstance() {
        if (instance == null) {
            synchronized (PerRequestRuntimeManager.class) {
                if (instance == null) {
                    instance = new PerRequestRuntimeManager();
                }
            }
        }
        return instance;
    }

    public RuntimeManager getRuntimeManager() {
        logger.debug("inside getRuntimeManager - per request");
        UserGroupCallback userGroupCallback = new LDAPUserGroupCallback();
        RuntimeEnvironmentBuilder envBuilder = RuntimeEnvironmentBuilder.getDefault();
        envBuilder.entityManagerFactory(Persistence.createEntityManagerFactory("org.jbpm.persistence.jpa"));
        envBuilder.userGroupCallback(userGroupCallback);
        envBuilder.addAsset(ResourceFactory.newClassPathResource("bpmn/cleanup.bpmn"), ResourceType.BPMN2);

        RuntimeEnvironment environment = envBuilder.get();

        logger.debug("Runtime Environment ..");
        runtimeManager = RuntimeManagerFactory.Factory.get().newPerRequestRuntimeManager(environment);

        getRuntime();

        return runtimeManager;
    }

    public synchronized RuntimeEngineImplDecorator getRuntime() {
        if (runtimeManager != null) {
            logger.debug("got runtime !! ");
            return new RuntimeEngineImplDecorator(runtimeManager, runtimeManager.getRuntimeEngine(EmptyContext.get()));
        }
        logger.debug("failed to get runtime..");
        return null;
    }

    public void disposeRuntime(RuntimeEngineImplDecorator runtimeDecorator) {
        logger.info("Disposing runtime..");
        if (runtimeDecorator != null && runtimeDecorator.getRuntimeManager() != null) {
            runtimeDecorator.getRuntimeManager().disposeRuntimeEngine(runtimeDecorator.getRuntimeEngine());
        } else {
            logger.info("Could not dispose runtime engine");
        }
    }

    public KieSession getKieSession(RuntimeEngine runtime) {
        registerWorkItemHandlers(runtime.getKieSession());
        return runtime.getKieSession();
    }

    private void registerWorkItemHandlers(KieSession kieSession) {
        logger.debug("Registering workitems and listners..");

    }

    public TaskService getTaskService(RuntimeEngine runtime) {
        registerWorkItemHandlers(runtime.getKieSession());
        return runtime.getTaskService();
    }

    public InternalTaskService getInternalTaskService(RuntimeEngine runtime) {
        registerWorkItemHandlers(runtime.getKieSession());
        return (InternalTaskService) runtime.getTaskService();
    }

    private EntityManagerFactory newPersistenceUnit() {
        return Persistence.createEntityManagerFactory("org.jbpm.persistence.jpa");
    }

    public JPAAuditLogService getAuditLogService() {
        logger.info("Producing auditLogService");
        return new JPAAuditLogService(newPersistenceUnit());
    }

    public void disposeAuditLogService(JPAAuditLogService auditLogService) {
        logger.info("Disposing auditLogService");
        auditLogService.dispose();
    }

}
